{
    SAVEMACS.EXE by Frank Fischer 7/14/91
    Given to the public domain.

    Usage: For the DOS 5.0 DOSKEY program.
           - Put SAVEMACS.EXE either in the root directory of C-drive
             or in another directory which is in your path.
           - Put the following line in you AUTOEXEC.BAT file:
                    CALL MACROS
           - If you reboot before using SAVEMACS, then the line in your
             AUTOEXEC.BAT file to call the MACROS.BAT file will simply
             do nothing.
           - If you execute SAVEMACS without DOSKEY installed or without
             having any current macros, then the MACROS.BAT file created
             in your root directory will not have any lines.
           - If you execute SAVEMACS with DOSKEY installed and at least one
             macro installed, then a file will be created in your root
             directory containing a line for every one of your current
             macros which will install the macros when your autoexec.bat
             file runs the next time you boot up.  Thus you can play around
             with deleting and adding macros, then call SAVEMACS and your
             current macros will be made the ones that get installed the
             next time you boot up.
}

{$A+,B-,D+,E-,F-,G-,I+,L+,N+,O-,R-,S+,V+,X-}
{$M 16384,0,0}

program savemacs;

uses
  dos, crt;

var
  macrosbat          : text;
  tempfile           : text;
  tempstr            : string;
  doskey_path        : pathstr;
  i                  : integer;
  permission_granted : boolean;
  f                  : file;

function fileexists(FileName: string) : Boolean;
  { Returns True if file exists; otherwise,
    it returns False. Closes the file if
    it exists. }
  begin
    {$I-}
    Assign(f, FileName);
    Reset(f);
    Close(f);
    {$I+}
    FileExists := (IOResult = 0) and
      (FileName <> '');
  end;  { fileexists }

procedure ask_permission;
  var
    ch : char;
  begin
    if not permission_granted then
      begin
        writeln;
        writeln('You have a file called "C:\MACROS.BAT"');
        writeln('  which was not created by this program.');
        write  ('Do you want to overwrite it (Y/N)?');
        ch := readkey;
        if ch = #0 then ch := readkey;
        writeln;
        if upcase(ch) <> 'Y' then
          begin
            write  ('SAVEMACS Aborted.');
            halt;
          end
        else
          permission_granted := true;
      end;
  end; {ask_permission}

function installed : boolean;
  var
    tempfile : text;
    mem_path           : pathstr;
  begin
   installed := false;
   mem_path := fsearch('MEM.EXE',getenv('PATH'));
   if mem_path = '' then
     begin
       writeln;
       writeln
       ('You don''t seem to have DOS 5.0 external commands at your disposal.');
       write  ('SAVEMACS Aborted.');
       halt;
     end;
   swapvectors;
   exec(getenv('COMSPEC'),' /C MEM /C>C:\$TEMP.T$$');
   swapvectors;
   assign(tempfile,'C:\$TEMP.T$$');
   reset(tempfile);
   while not eof(tempfile) do
     begin
       readln(tempfile,tempstr);
       if pos('DOSKEY',tempstr) <> 0 then
         installed := true;
     end;
   close(tempfile);
   erase(tempfile);
  end; {installed}

begin {main block}
  assign(input,'');  reset(input);
  assign(output,''); rewrite(output);
  writeln('SAVEMACS 1.0 by Frank Fischer 7/14/91');
  writeln('   Given to the public domain.');
  writeln('   Macro Installation for the DOS 5.0 DOSKEY program.');
  writeln('   Creates a Macros.Bat file containing the lines to');
  writeln('     install your current macros in the root directory');
  writeln('     of your C-drive.');
  writeln('   Correctly handles macros using the DOS "for" command by');
  writeln('     inserting a second "%" in front of the replaceble varibles.');
  writeln('   Correctly handles macros containing control characters.');
  writeln('   Put the following line in your Autoexec.Bat file:');
  writeln('        CALL C:\MACROS');
  writeln('   Be sure to put the call to Macros.Bat after the line');
  writeln('     in your Autoexec.Bat where DOSKEY is installed,');
  writeln('     and after the line installing your PATH.');
  doskey_path := fsearch('DOSKEY.COM',getenv('PATH'));
  if doskey_path = '' then
    begin
      writeln;
      writeln('DOSKEY.COM is not in a directory in your path.');
      writeln('Either you do not have DOS 5.0 or you have put');
      writeln('    DOSKEY.COM somewhere inaccessable.');
      write  ('SAVEMACS Aborted.');
      halt;
    end;
  if not installed then
    begin
      writeln;
      writeln('DOSKEY is not installed.');
      write  ('SAVEMACS Aborted.');
      halt;
    end;
  permission_granted := false;
  if fileexists('C:\MACROS.BAT') then
    begin
      assign(macrosbat,'C:\MACROS.BAT');
      reset(macrosbat);
      if eof(macrosbat) then
        ask_permission
      else
        begin
          readln(macrosbat,tempstr);
          if tempstr <> '@echo off' then ask_permission;
        end;
      if eof(macrosbat) then
        ask_permission
      else
        begin
          readln(macrosbat,tempstr);
          if tempstr <> 'echo DOSKEY macros now being installed...' then
            ask_permission;
        end;
      while not eof(macrosbat) do
        begin
          readln(macrosbat,tempstr);
          if copy(tempstr,1,7) <> 'DOSKEY ' then ask_permission;
        end;
      close(macrosbat);
    end;
  write ('C:\MACROS.BAT being created...');
  doskey_path := copy(doskey_path,1,pos('.',doskey_path)-1);
  swapvectors;
  exec(getenv('COMSPEC'),' /C '+doskey_path+' /MACROS>C:\MACROS.BAT');
  swapvectors;
  assign(tempfile,'C:\MACROS.BAT');
  reset(tempfile);
  assign(macrosbat,'C:\$T$E$M$P.$T$');
  rewrite(macrosbat);
  writeln(macrosbat,'@echo off');
  writeln(macrosbat,'echo DOSKEY macros now being installed...');
  close(macrosbat);
  assign(f,'C:\MACROS.BAT');
  reset(f,1);
  if filesize(f) = 2 then
    begin
      close(f);
      writeln;
      write  ('You have no current macros!');
    end
  else
    begin
      close(f);
      append(macrosbat);
      while not eof(tempfile) do
      begin
        readln(tempfile,tempstr);
        i := 0;
            while i <= length(tempstr) do
            begin
              inc(i);
              if tempstr[i] = '%' then
                begin
                  insert('%',tempstr,i);
                  inc(i);
                end;
              if tempstr[i] = '^' then
                begin
                  insert(chr(ord(tempstr[i+1])-64),tempstr,i);
                  delete(tempstr,i+1,2);
                end;
            end;
            writeln(macrosbat,'DOSKEY ',tempstr);
      end;
      close(macrosbat);
    end;
  close(tempfile);
  erase(tempfile);
  rename(macrosbat,'C:\MACROS.BAT');
end.  {main block}
